/*
 * Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
 * Copyright (C) 2010 - DIGITEO - Allan SIMON
 * 
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at    
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 *
 */

package org.scilab.modules.metanet.actions;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import ncsa.hdf.hdf5lib.exceptions.HDF5Exception;
import ncsa.hdf.hdf5lib.exceptions.HDF5JavaException;
import ncsa.hdf.hdf5lib.exceptions.HDF5LibraryException;

import org.scilab.modules.action_binding.highlevel.ScilabInterpreterManagement;
import org.scilab.modules.action_binding.highlevel.ScilabInterpreterManagement.InterpreterException;
import org.scilab.modules.graph.ScilabGraph;
import org.scilab.modules.graph.actions.base.DefaultAction;
import org.scilab.modules.gui.menuitem.MenuItem;
import org.scilab.modules.gui.pushbutton.PushButton;
import org.scilab.modules.hdf5.H5ScilabConstant;
import org.scilab.modules.hdf5.read.H5Read;
import org.scilab.modules.metanet.edge.BasicEdge;
import org.scilab.modules.metanet.graph.MetanetDiagram;
import org.scilab.modules.metanet.utils.MetanetFileType;
import org.scilab.modules.metanet.utils.MetanetMessages;
import org.scilab.modules.types.ScilabString;

import com.mxgraph.model.mxGraphModel;

public class CircuitAction extends DefaultAction {
    private static final long serialVersionUID = 1L;

    public static final String NAME = MetanetMessages.CIRCUIT;
    public static final String SMALL_ICON = "";
    public static final int MNEMONIC_KEY = 0;
    public static final int ACCELERATOR_KEY = 0;

    private MetanetDiagram diagram;

    /**
     * Constructor
     * 
     * @param scilabGraph
     *            associated Scilab Graph
     */
    public CircuitAction(ScilabGraph scilabGraph) {
        super(scilabGraph);
    }

    /**
     * Create a menu to add in Scilab Graph menu bar
     * 
     * @param scilabGraph
     *            associated Scilab Graph
     * @return the menu
     */
    public static MenuItem createMenu(ScilabGraph scilabGraph) {
        return createMenu(scilabGraph, CircuitAction.class);
    }

    /**
     * Create a button to add in Scilab Graph tool bar
     * 
     * @param scilabGraph
     *            associated Scilab Graph
     * @return the button
     */
    public static PushButton createButton(ScilabGraph scilabGraph) {
        return createButton(scilabGraph, CircuitAction.class);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        diagram = ((MetanetDiagram) getGraph(e));
        if (!diagram.isDirected()) {
            return;
        }
        updateUI(true);

        try {

            final File tempOutput = File.createTempFile(MetanetFileType.XGRAPH.getExtension(), MetanetFileType.HDF5.getExtension(),
            // new File(System.getenv(MetanetConstants.TMPDIR)));
                    new File("/tmp"));
            final File tempInput = File.createTempFile(MetanetFileType.XGRAPH.getExtension(), MetanetFileType.HDF5.getExtension(),
            // new File(System.getenv(MetanetConstants.TMPDIR)));
                    new File("/tmp"));

            diagram.dumpToHdf5File(tempOutput.getAbsolutePath());

            String command = "import_from_hdf5(\"" + tempOutput.getAbsolutePath() + "\");" + "edgeIds = edges_2_ids(circuit(g),g);" + "export_to_hdf5(\""
                    + tempInput.getAbsolutePath() + "\",'edgeIds');" + "deletefile(\"" + tempOutput.getAbsolutePath() + "\");";
            try {
                ScilabInterpreterManagement.asynchronousScilabExec(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent arg0) {
                        ScilabString data = new ScilabString();

                        try {
                            int fileId = H5Read.openFile(tempInput.getAbsolutePath());

                            // if scilab doesn't return an empty matrix
                            if (!H5Read.getRootType(fileId).equals(H5ScilabConstant.SCILAB_CLASS_DOUBLE)) {
                                H5Read.readDataFromFile(fileId, data);

                                String[] path = data.getData()[0];
                                BasicEdge[] pathEdges = new BasicEdge[path.length];

                                for (int i = 0; i < path.length; ++i) {
                                    pathEdges[i] = (BasicEdge) ((mxGraphModel) diagram.getModel()).getCell(path[i]);
                                }

                                diagram.setSelectionCells(pathEdges);
                                diagram.refresh();
                            }
                            H5Read.closeFile(fileId);

                        } catch (HDF5LibraryException e) {
                            updateUI(false);
                            System.err.println("An error occurred while working with the HDF5 library " + e.getLocalizedMessage());
                        } catch (HDF5JavaException e) {
                            updateUI(false);
                            System.err.println("A HDF5 loading error occurred: Error: " + e.getLocalizedMessage());
                        } catch (HDF5Exception e) {
                            updateUI(false);
                            System.err.println("A HDF5 loading error occurred: Error: " + e.getLocalizedMessage());
                        }

                        updateUI(false);
                        data.toString();
                    }
                }, command);
            } catch (InterpreterException e1) {
                e1.printStackTrace();
                updateUI(false);
            }
        } catch (IOException e1) {
            e1.printStackTrace();
            updateUI(false);
        }
    }

    /**
     * Update the UI depending on the action selected or not
     * 
     * @param started
     *            the started status
     */
    public void updateUI(boolean started) {

        ((MetanetDiagram) getGraph(null)).setReadOnly(started);

        if (started) {
            ((MetanetDiagram) getGraph(null)).info("searching for a circuit...");
        } else {
            ((MetanetDiagram) getGraph(null)).info(MetanetMessages.EMPTY_INFO);
        }
    }

}
